# ********************************************************************************
# +----------------------------------------------------------------------------
# | File : Unpublish.ps1      Updated 10/31
# +----------------------------------------------------------------------------
# ********************************************************************************

<#
.SYNOPSIS
Unpublish script to remove the dependencies of Acrobat created as a part of publishing the package to the machine.

.DESCRIPTION
.\Unpublish.ps1 [-CreateLog] [-LogFile <Path to Logfile>] [-ChannelName {"DC|2015|2017"}]

.EXAMPLE
.\Unpublish.ps1 -ChannelName "2015|2017"
Runs Unpublish.ps1 for Classic track

.EXAMPLE
.\Unpublish.ps1 -CreateLog -LogFile testresults.log -ChannelName "DC"
Runs Unpublish.ps1 for Continuous track and create logs in testresults.log
#>

param(
#Identifies the delivery track of the application, should match the track from which package is created. Value is either "DC" or "2015" or "2017".
[string]$ChannelName="2015",
#Specifies the filepath to which script logs will be written.
[string]$LogFile="$env:Temp\Acrobat-Appv.log",
#switched parameter needs to be provided if FRL licensed AppV package. 
[switch]$Frl,
#Switched parameter that generates Script Logs on the client machine, at the location described by -LogFile parameter.
[switch]$CreateLog)
# +----------------------------------------------------------------------------
# | FUNCTIONS
# | 
# | description of function, parameters and what it returns (if any)                                                                                         
# +----------------------------------------------------------------------------
function ShowHeading {
##-----------------------------------------------------------------------
## Function: ShowHeading
## Purpose: Use to write a header for the script
##-----------------------------------------------------------------------
        Log "---------------------------------------------------------"  
        Log "-                      Unpublish.ps1                    -" 
        Log "-        Unpublish Time Script for Acrobat App-V        -"
        Log "---------------------------------------------------------" 
}
function Log {
##-----------------------------------------------------------------------
## Function: Log
## Purpose: Used to write output to a log file
##-----------------------------------------------------------------------
    param ([string]$argOut)
    $kDateAndTime = get-date -uFormat "%d-%m-%Y %H:%M"
    $kOut = ($kDateAndTime+ ": " + $argOut)
    If ($LogFile -and $CreateLog) { "$kOut" | out-file -filepath $LogFile -append }
}
function PrintAndLog {
##-----------------------------------------------------------------------
## Function: PrintAndLog
## Purpose: Used to write output to the screen and Log it to a File
##-----------------------------------------------------------------------
    param ([string]$argOut)
    write-host $argOut
    If ($CreateLog) { Log $argOut }
}
##---------------------------------------------------------------------------------
## Function: RemoveAppVRegistry
## Purpose: This function is used to remove appv key created at publish time
##---------------------------------------------------------------------------------
function RemoveAppVRegistry
{
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog "Removing appv registry key"
    PrintAndLog "-------------------------------------------------------------------------------"
    # Registries that are made in Native Hive
    $kAcrobatAppVKey = "HKLM:\Software\Adobe\Adobe Acrobat\$ChannelName\appv"
    If (Test-Path $kAcrobatAppvKey){
        PrintAndLog "DEBUG: [Native] - Removing $kAcrobatAppVKey"
        Remove-Item $kAcrobatAppVKey
    }  
    # + Registries if made in Wow Hive
    If([System.Environment]::Is64BitProcess){
        $kAcrobatWow64AppVHive = "HKLM:\Software\Wow6432Node\Adobe\Adobe Acrobat\$ChannelName\appv"
        If (Test-Path $kAcrobatWow64AppvHive) {
            PrintAndLog "DEBUG: [Wow6432] - $kAcrobatWow64AppVHive is available"
            Remove-Item $kAcrobatWow64AppVHive
        }
    }
}
##---------------------------------------------------------------------------------
## Function: DeactivateAcrobat
## Purpose: This function is responsible for De-Serializing Acrobat on Client
##---------------------------------------------------------------------------------
function DeactivateAcrobat
{
    param([String]$argProgramFilesCommon)
    
    $kSupportedLEIDs = @()
    If($ChannelName -eq "2015") {
        $kCurrentLEID = "V7{}AcrobatESR-12-Win-GM"  # For Classic 2015 Retail and Volume Serials
    }
    ElseIf($ChannelName -eq "2017"){
    $kCurrentLEID = "V7{}AcrobatESR-17-Win-GM"      # For Classic 2017 Retail and Volume Serials
    }
	ElseIf($ChannelName -eq "2020"){
    $kCurrentLEID = "V7{}AcrobatESR-20-Win-GM"      # For Classic 2020 Retail and Volume Serials
    }
     Else {
        $kCurrentLEID = "V7{}AcrobatCont-12-Win-GM" # For Continuous Retail and Volume Serials
    }
    
    $kSupportedLEIDs += $kCurrentLEID
    $kSupportedLEIDs += "V6{}AcrobatPro-AS2-Win-GM"     # For Pro Subscription
    $kSupportedLEIDs += "V6{}AcrobatStd-AS2-Win-GM"     # For Standard Subscription
    $kSupportedLEIDs += "V7{}AcrobatETLA-12-Win-GM"     # For ETLA Serial Numbers
    
    $kPayloadCode = "{AC76BA86-1033-FFFF-7760-000000000007}"
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog "Deactivating Virtual Adobe Acrobat DC using LEIDs: $kSupportedLEIDs"
    PrintAndLog "-------------------------------------------------------------------------------"
    
    foreach ($kAcrobatLEID in $kSupportedLEIDs) {
        $kDeactivationCmdLine = "adobe_prtk.exe --tool=UnSerialize --leid=" + $kAcrobatLEID + " --deactivate --force --removeSWTag"
        PrintAndLog "DEBUG: Running $kDeactivationCmdLine"
        & cmd.exe /c "$kDeactivationCmdLine"
        PrintAndLog "DEBUG: Adobe_PRTK returned $LASTEXITCODE"
    }
    PrintAndLog "DEBUG: Licensing Script execution completed"
    
    PrintAndLog "DEBUG: Trying to remove $argProgramFilesCommon\PCF\$kPayloadCode.$kCurrentLEID.xml"
    if(test-path "$argProgramFilesCommon\PCF\$kPayloadCode.$kCurrentLEID.xml")
    {
        Remove-Item "$argProgramFilesCommon\PCF\$kPayloadCode.$kCurrentLEID.xml"
    }
    PrintAndLog "DEBUG: Depedent Files removed from the system"
    PrintAndLog "-------------------------------------------------------------------------------"
	
	if ($Frl)
	{
		& cmd.exe /c "adobe-licensing-toolkit.exe -p -u -f ngl-preconditioning-data.json"
	}
}
##---------------------------------------------------------------------------------
## Function: RemoveUpdaterKey
## Purpose: This function removes the Updater key in App-V Acrobat
##---------------------------------------------------------------------------------
function RemoveUpdaterKey
{
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog "Removing Policies key to shutdown bUpdater"
    PrintAndLog "-------------------------------------------------------------------------------"
            
    # Registries that are made in the native hive
    $kAcrobatLockDownHive = "HKLM:\Software\Policies\Adobe\Adobe Acrobat\$ChannelName\FeatureLockDown"
    If(Test-Path $kAcrobatLockDownHive) {
        PrintAndLog "DEBUG: [Native] - FeatureLockDown Key available"
        $objAcrobatKey = Get-Item -LiteralPath $kAcrobatLockDownHive
        If ($objAcrobatKey.GetValue("bUpdater", $null) -ne $null) {
            PrintAndLog "DEBUG: [Native] - bUpdater Key present"
            Remove-ItemProperty -Path $kAcrobatLockDownHive -Name bUpdater -Force
            PrintAndLog "DEBUG: [Native] - bUpdater Key removed"
        } Else {
            PrintAndLog "DEBUG: [Native] - bUpdater Key not present"
        }
    } Else {
        PrintAndLog "DEBUG: [Native] - FeatureLockDown Key not present"
    }
        
    If([System.Environment]::Is64BitProcess) {
        # + Registries that are made in the Wow6432 Hive
        $kAcrobatLockDownWowHive = "HKLM:\Software\Wow6432Node\Policies\Adobe\Adobe Acrobat\$ChannelName\FeatureLockDown"
        If(Test-Path $kAcrobatLockDownWowHive) {
            PrintAndLog "DEBUG: [Wow6432] - FeatureLockDown Key available"
            $objAcrobatWowKey = Get-Item -LiteralPath $kAcrobatLockDownWowHive
            If ($objAcrobatWowKey.GetValue("bUpdater", $null) -ne $null) {
                PrintAndLog "DEBUG: [Wow6432] - bUpdater Key present"
                Remove-ItemProperty -Path $kAcrobatLockDownWowHive -Name bUpdater -Force
                PrintAndLog "DEBUG: [Wow6432] - bUpdater Key removed"
            } Else {
                PrintAndLog "DEBUG: [Wow6432] - bUpdater Key not present"
            }
        } Else {
            PrintAndLog "DEBUG: [Wow6432] - FeatureLockDown Key not present"
        }
    }
}
##---------------------------------------------------------------------------------
## Function: RemoveNotificationKey
## Purpose: This function removes the Notification key in App-V Acrobat
##---------------------------------------------------------------------------------
function RemoveNotificationKey
{
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog "Removing Policies key to shutdown cServices\bToggleNotifications"
    PrintAndLog "-------------------------------------------------------------------------------"
            
    # Registries that are made in the native hive
    $kAcrobatLockDownHive = "HKLM:\Software\Policies\Adobe\Adobe Acrobat\$ChannelName\FeatureLockDown\cServices"
    If(Test-Path $kAcrobatLockDownHive) {
        PrintAndLog "DEBUG: [Native] - cServices FeatureLockDown Key available"
        $objAcrobatKey = Get-Item -LiteralPath $kAcrobatLockDownHive
        If ($objAcrobatKey.GetValue("bToggleNotifications", $null) -ne $null) {
            PrintAndLog "DEBUG: [Native] - bToggleNotifications Key present"
            Remove-ItemProperty -Path $kAcrobatLockDownHive -Name bToggleNotifications -Force
            PrintAndLog "DEBUG: [Native] - bToggleNotifications Key removed"
        } Else {
            PrintAndLog "DEBUG: [Native] - bToggleNotifications Key not present"
        }
    } Else {
        PrintAndLog "DEBUG: [Native] - cServices FeatureLockDown Key not present"
    }
        
    If([System.Environment]::Is64BitProcess) {
        # + Registries that are made in the Wow6432 Hive
        $kAcrobatLockDownWowHive = "HKLM:\Software\Wow6432Node\Policies\Adobe\Adobe Acrobat\$ChannelName\FeatureLockDown\cServices"
        If(Test-Path $kAcrobatLockDownWowHive) {
            PrintAndLog "DEBUG: [Wow6432] - cServices FeatureLockDown Key available"
            $objAcrobatWowKey = Get-Item -LiteralPath $kAcrobatLockDownWowHive
            If ($objAcrobatWowKey.GetValue("bToggleNotifications", $null) -ne $null) {
                PrintAndLog "DEBUG: [Wow6432] - bToggleNotifications Key present"
                Remove-ItemProperty -Path $kAcrobatLockDownWowHive -Name bToggleNotifications -Force
                PrintAndLog "DEBUG: [Wow6432] - bToggleNotifications Key removed"
            } Else {
                PrintAndLog "DEBUG: [Wow6432] - bToggleNotifications Key not present"
            }
        } Else {
            PrintAndLog "DEBUG: [Wow6432] - cServices FeatureLockDown Key not present"
        }
    }
}
##---------------------------------------------------------------------------------
## Function: RemoveSandboxingKey
## Purpose: This function removes the Sandboxing key in App-V Acrobat
##---------------------------------------------------------------------------------
function RemoveSandboxingKey
{
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog "Removing Policies key to shutdown bProtectedMode"
    PrintAndLog "-------------------------------------------------------------------------------"
            
    # Registries that are made in the native hive
    $kAcrobatLockDownHive = "HKLM:\Software\Policies\Adobe\Adobe Acrobat\$ChannelName\FeatureLockDown"
    If(Test-Path $kAcrobatLockDownHive) {
        PrintAndLog "DEBUG: [Native] ] - FeatureLockDown Key available"
        $objAcrobatKey = Get-Item -LiteralPath $kAcrobatLockDownHive
        If ($objAcrobatKey.GetValue("bProtectedMode", $null) -ne $null) {
            PrintAndLog "DEBUG: [Native] - bProtectedMode Key present"
            Remove-ItemProperty -Path $kAcrobatLockDownHive -Name bProtectedMode -Force
            PrintAndLog "DEBUG: [Native] - bProtectedMode Key removed"
        } Else {
            PrintAndLog "DEBUG: [Native] - bProtectedMode Key not present"
        }
    } Else {
        PrintAndLog "DEBUG: [Native] -  FeatureLockDown Key not present"
    }
        
    If([System.Environment]::Is64BitProcess) {
        # + Registries that are made in the Wow6432 Hive
        $kAcrobatLockDownWowHive = "HKLM:\Software\Wow6432Node\Policies\Adobe\Adobe Acrobat\$ChannelName\FeatureLockDown"
        If(Test-Path $kAcrobatLockDownWowHive) {
            PrintAndLog "DEBUG: [Wow6432] FeatureLockDown Key available"
            $objAcrobatWowKey = Get-Item -LiteralPath $kAcrobatLockDownWowHive
            If ($objAcrobatWowKey.GetValue("bProtectedMode", $null) -ne $null) {
                PrintAndLog "DEBUG: [Wow6432] - bProtectedMode Key present"
                Remove-ItemProperty -Path $kAcrobatLockDownWowHive -Name bProtectedMode -Force
                PrintAndLog "DEBUG: [Wow6432] - bProtectedMode Key removed"
            } Else {
                PrintAndLog "DEBUG: [Wow6432] - bProtectedMode Key not present"
            }
        } Else {
            PrintAndLog "DEBUG: [Wow6432] -  FeatureLockDown Key not present"
        }
    }
}
##-----------------------------------------------------------------------
## Function: Usage
## Purpose: This function is used to describe how to use this script
##-----------------------------------------------------------------------
function Usage
{
@"
    Usage: 
        
     .\Unpublish.ps1 [-CreateLog] [-LogFile <Path to Logfile>] [-ChannelName {"DC|2015|2017|2020"}]
     Runs the Unpublish script to remove the dependencies of Acrobat created as a part of publishing the package to the machine.
     e.g. .\Unpublish.ps1 -CreateLog -LogFile testing.log -ChannelName "2015"
        
"@
    return
}
# +----------------------------------------------------------------------------
# | MAIN ROUTINE                                                                                         
# +----------------------------------------------------------------------------
function Main()
{
    ShowHeading
  
    # Step 1: Perform the registry creation (appv + policies)
    RemoveAppVRegistry
    RemoveUpdaterKey
    RemoveNotificationKey
    RemoveSandboxingKey
        
    #Step 2: Perform Licensing Workflows
    If ([System.Environment]::Is64BitProcess) {
        $kProgramFiles = "C:\Program Files (x86)"
    } Else {
        $kProgramFiles = "C:\Program Files"
    }
    DeactivateAcrobat "$kProgramFiles\Common Files\Adobe"
}

Main
